@@ -38,7 +38,7 @@ for manpage in glob.glob("../src/*.[18]"):
 ADMONITIONS = collections.OrderedDict([
     ("error", ["(BUG) In- or external bug that *should be fixed* eventually", []]),
     ("attention", ["(TODO) Something that *needs attention* eventually", []]),
-    ("important", ["(NOTE) Something not obvious you should be aware of", []]),
+    ("note", ["(NOTE) Something not obvious you should be aware of", []]),
     ("tip", ["(FAQ) Free tip", []]),
 ])
 ADMONITIONS_FILE = "auto_admonitions.rst"
