@@ -14,7 +14,6 @@ import sys
 import inspect
 import collections
 import logging
-import json
 import argparse
 import re
 import glob
@@ -344,8 +343,6 @@ class Call():
     CONFIRM = False
     CUSTOM_HTML_SNIPPET = None
 
-    PLAIN = "__plain__"
-
     @classmethod
     def name(cls):
         return mini_buildd.misc.Snake(cls.__name__).from_camel()
@@ -411,12 +408,16 @@ class Call():
         self.request = request
 
     @classmethod
-    def json_pretty(cls, result):
-        return json.dumps(result, indent=2)
+    def get_plain(cls, result, force_json=False):
+        """Get ``str`` result (non-``str`` results get json pretty-formatted)"""
+        return mini_buildd.misc.json_pretty(result) if (force_json or not isinstance(result, str)) else result
 
-    @classmethod
-    def get_plain(cls, result):
-        return result.get(cls.PLAIN, cls.json_pretty(result))
+    def plain(self, force_json=False):
+        return self.get_plain(self.result, force_json=force_json)
+
+    def json_pretty(self):
+        """For (arg-less) use in templates only"""
+        return self.plain(force_json=True)
 
     @classmethod
     def parse_command_line(cls, command_line):
@@ -451,15 +452,21 @@ class Call():
         user = self.request.user if self.AUTH is not mini_buildd.config.AUTH_NONE and self.request is not None and self.request.user.is_authenticated else None
         return self.command_line(full=True, with_user=user is not None, user=user)
 
-    def http_args(self, with_confirm=False):
-        http_args = {"confirm": self.name()} if with_confirm else {}
+    def http_args(self, with_confirm=False, with_output=None):
+        http_args = {}
         for arg in self.args.values():
             if arg.given is not None:
                 http_args[arg.identity] = arg.strgiven()
+
+        if with_confirm:
+            http_args["confirm"] = self.name()
+        if with_output is not None:
+            http_args["output"] = with_output
+
         return http_args
 
-    def plain(self):
-        return self.get_plain(self.result)
+    def url(self):
+        return mini_buildd.http_endpoint().geturl(path=mini_buildd.config.URIS["api"]["view"].join(self.name() + "/"), query=self.http_args(with_output="html"))
 
     @abc.abstractmethod
     def run(self):
@@ -497,16 +504,6 @@ class _Confirm:
     CONFIRM = True
 
 
-class _Show:
-    CUSTOM_HTML_SNIPPET = "show"
-
-    @classmethod
-    def show(cls, source, codename):
-        show = Show(source=source, codenames=codename)
-        show.run()
-        return show.result
-
-
 #: API calls
 @_pimpdoc()
 class Status(Call):
@@ -554,7 +551,7 @@ class PubKey(Call):
     """
 
     def run(self):
-        self.result[self.PLAIN] = mini_buildd.get_daemon().gnupg.pub_key
+        self.result = mini_buildd.get_daemon().gnupg.pub_key
 
 
 @_pimpdoc()
@@ -570,13 +567,12 @@ class DputConf(Call):
     """
 
     def run(self):
-        self.result[self.PLAIN] = mini_buildd.get_daemon().model.mbd_get_dput_conf()
+        self.result = mini_buildd.get_daemon().model.mbd_get_dput_conf()
 
         # Optionally add local custom config
         try:
             with mini_buildd.fopen(mini_buildd.config.ROUTES["etc"].path.join("dput.cf")) as extra_conf:
-                self.result[self.PLAIN] += "\n"
-                self.result[self.PLAIN] += extra_conf.read()
+                self.result += extra_conf.read()
         except FileNotFoundError:
             pass
 
@@ -616,10 +612,10 @@ class SourcesList(Call):
                     for rollback in self.args["rollbacks"].range(s):
                         sources_list.append(r.mbd_get_apt_line(d, s, snapshot=self.args["snapshot"].value(), rollback=rollback))
 
-        self.result[self.PLAIN] = sources_list.get(self.args["types"].value(),
-                                                   mirror=self.args["mirror"].value(),
-                                                   options=self.args["options"].value(),
-                                                   with_comment=self.args["with_comment"].value())
+        self.result = sources_list.get(self.args["types"].value(),
+                                       mirror=self.args["mirror"].value(),
+                                       options=self.args["options"].value(),
+                                       with_comment=self.args["with_comment"].value())
 
 
 @_pimpdoc()
@@ -678,21 +674,17 @@ class Debdiff(Call):
         repositories_path = mini_buildd.config.ROUTES["repositories"].path
         dsca = repository.mbd_dsc_pool_path(self.args["source"].value(), self.args["versiona"].value())
         dscb = repository.mbd_dsc_pool_path(self.args["source"].value(), self.args["versionb"].value())
-        self.result[self.PLAIN] = mini_buildd.call.Call(["debdiff", repositories_path.join(dsca), repositories_path.join(dscb)]).stdout
+        self.result = mini_buildd.call.Call(["debdiff", repositories_path.join(dsca), repositories_path.join(dscb)]).stdout
 
 
 @_pimpdoc()
-class Migrate(_Staff, _Confirm, _Show, Call):
+class Migrate(_Staff, _Confirm, Call):
     """
     Migrate source package
 
     Migrates a source package along with all its binary packages. If
     run for a rollback distribution, this will perform a rollback
     restore.
-
-    JSON Result (dict):
-      __plain__: string: textual migration log.
-
     """
 
     @classmethod
@@ -711,18 +703,14 @@ class Migrate(_Staff, _Confirm, _Show, Call):
                                        full=self.args["full"].value(),
                                        rollback=self.args["distribution"].dist().rollback_no,
                                        version=version if version else None)
-        self.result = self.show(self.args["source"].value(), self.args["distribution"].dist().codename)
 
 
 @_pimpdoc()
-class Remove(_Admin, _Confirm, _Show, Call):
+class Remove(_Admin, _Confirm, Call):
     """
     Remove source package
 
     Removes a source package along with all its binary packages.
-
-    JSON Result (dict):
-      __plain__: string: textual removal log.
     """
 
     @classmethod
@@ -741,12 +729,11 @@ class Remove(_Admin, _Confirm, _Show, Call):
                                       rollback=self.args["distribution"].dist().rollback_no,
                                       version=version if version else None,
                                       without_rollback=self.args["without_rollback"].value())
-        self.result = self.show(self.args["source"].value(), self.args["distribution"].dist().codename)
 
 
 PORT_RESULT_DOC = (
     "JSON Result (dict):",
-    "  uploaded: list of triples of all items uploaded",
+    "  list of dicts of all items uploaded",
 )
 
 
@@ -771,14 +758,14 @@ class Port(_Staff, _Running, _Confirm, Call):
         yield UploadOptions(["--options", "-O"])
 
     def run(self):
-        self.result["uploaded"] = []
+        self.result = []
         version = self.args["version"].value()
         for to_diststr in self.args["to_distributions"].value():
-            self.result["uploaded"].append(mini_buildd.package.port(self.args["source"].value(),
-                                                                    self.args["from_distribution"].value(),
-                                                                    to_diststr,
-                                                                    version=version if version else None,
-                                                                    options=self.args["options"].value().split("|")))
+            self.result.append(mini_buildd.package.port(self.args["source"].value(),
+                                                        self.args["from_distribution"].value(),
+                                                        to_diststr,
+                                                        version=version if version else None,
+                                                        options=self.args["options"].value().split("|")))
 
 
 @_pimpdoc(PORT_RESULT_DOC)
@@ -798,12 +785,12 @@ class PortExt(_Staff, _Running, _Confirm, Call):
         yield UploadOptions(["--options", "-O"])
 
     def run(self):
-        self.result["uploaded"] = []
+        self.result = []
         for diststr in self.args["distributions"].value():
-            self.result["uploaded"].append(mini_buildd.package.port_ext(self.args["dsc"].value(),
-                                                                        diststr,
-                                                                        options=self.args["options"].value().split("|"),
-                                                                        allow_unauthenticated=self.args["allow_unauthenticated"].value()))
+            self.result.append(mini_buildd.package.port_ext(self.args["dsc"].value(),
+                                                            diststr,
+                                                            options=self.args["options"].value().split("|"),
+                                                            allow_unauthenticated=self.args["allow_unauthenticated"].value()))
 
 
 @_pimpdoc()
@@ -848,7 +835,7 @@ class Retry(_Staff, _Running, _Confirm, Call):
 
         # Should be save now to re-upload the changes files found in path
         mini_buildd.changes.Base(changes).upload(mini_buildd.get_daemon().model.mbd_get_ftp_endpoint(), force=True)
-        self.result["changes"] = os.path.basename(changes)
+        self.result = os.path.basename(changes)
 
 
 @_pimpdoc()
@@ -861,7 +848,7 @@ class Cancel(_Staff, _Running, _Confirm, Call):
 
     def run(self):
         mini_buildd.get_daemon().builder.cancel(self.args["bkey"].value(), f"{self.request.user}")
-        self.result["canceled"] = self.args["bkey"].value()
+        self.result = self.args["bkey"].value()
 
 
 @_pimpdoc()
@@ -922,7 +909,7 @@ class Subscribe(_Login, Call):
             package=self.sourcestr,
             distribution=self.distributionstr,
         )
-        self.result["subscribed"] = {"created": created, "source": s.package, "distribution": s.distribution}
+        self.result = {"created": created, "source": s.package, "distribution": s.distribution}
 
 
 @_pimpdoc()
@@ -932,11 +919,11 @@ class Unsubscribe(Subscribe):
     """
 
     def run(self):
-        self.result["unsubscribed"] = []
+        self.result = []
         for s in mini_buildd.mdls().subscription.Subscription.objects.filter(subscriber=self.request.user):
             if (self.sourcestr == s.package) and (self.distributionstr == s.distribution):
                 s.delete()
-                self.result["unsubscribed"].append({"source": s.package, "distribution": s.distribution})
+                self.result.append({"source": s.package, "distribution": s.distribution})
 
 
 @_pimpdoc()
@@ -1020,7 +1007,7 @@ class Handshake(Call):
         for r in mini_buildd.mdls().gnupg.Remote.objects.all():
             try:
                 r.mbd_verify(signed_message)
-                self.result[self.PLAIN] = mini_buildd.get_daemon().handshake_message()
+                self.result = mini_buildd.get_daemon().handshake_message()
                 LOG.debug(f"Remote handshake ok: '{r}': {r.key_long_id}: {r.key_name}")
                 return
             except Exception as e:
@@ -1075,7 +1062,7 @@ class SnapshotLs(_Running, Call):
 
     def run(self):
         diststr = self.args["distribution"].value()
-        self.result = {diststr: diststr2repository(diststr).mbd_reprepro.get_snapshots(diststr)}
+        self.result = diststr2repository(diststr).mbd_reprepro.get_snapshots(diststr)
 
 
 @_pimpdoc()
@@ -1113,12 +1100,16 @@ class Debmirror(_Admin, _Confirm, Call):
     only certain repos, only ``stable``, omit rollbacks) variant of your
     repo somewhere remote.
 
-    .. error:: :debbug:`819925`: ``apt update`` fails on ``experimental`` suites (contents not mirrored)
+    .. error:: debmirror (:debbug:`819925`): ``apt update`` fails on ``experimental`` suites (contents not mirrored)
 
       This happens only on systems where APT is configured to download contents, most likely just because
       ``apt-file`` is installed. So, the easiest workaround is::
 
         apt purge apt-file
+
+      Closest to an actual fix is to install debmirror variant '+abfixes' from
+      ``Hellfield Archive``, where the "bug" has been fixed (Dec 2022: at least
+      available for bullseye).
     """
 
     PROGRAM = "/usr/bin/debmirror"
@@ -1141,7 +1132,7 @@ class Debmirror(_Admin, _Confirm, Call):
                           default=Value.DEFAULT_DEBMIRROR_DESTINATION,
                           doc=("Mirror destination dir (``mirrordir`` in debmirror)\n"
                                "\n"
-                               "* ``{}`` will be replaced by repository identidy\n"
+                               "* ``{}`` will be replaced by repository identity\n"
                                "* Directory must be accessible by user ``mini-buildd``\n"
                                "\n"
                                "*BE CAREFUL* with this value -- anything in this directory will be *HAPPILY REPLACED* by the mirror only\n"))
@@ -1188,25 +1179,27 @@ class KeyringPackages(_Admin, _Running, _Confirm, Call):
         yield BoolArgument(["--without-migration", "-M"], doc="migrate packages")
 
     def run(self):
-        self.result["uploaded"] = []
+        self.result = []
         events = mini_buildd.events.Attach(mini_buildd.get_daemon().events)
 
         for diststr in self.args["distributions"].value():
             repository, distribution, suite = mini_buildd.mdls().repository.parse_diststr(diststr)
             if not suite.build_keyring_package:
                 raise mini_buildd.HTTPBadRequest(f"Keyring package to non-keyring suite requested (see 'build_keyring_package' flag): '{diststr}'")
-            self.result["uploaded"].append(mini_buildd.package.upload_template_package(mini_buildd.package.KeyringPackage(), diststr))
+            self.result.append(mini_buildd.package.upload_template_package(mini_buildd.package.KeyringPackage(), diststr))
 
-        uploaded = set(self.result["uploaded"])
-        while uploaded:
+        def unfinished():
+            return [upload for upload in self.result if "event" not in upload]
+
+        while unfinished():
             event = events.get()
-            for diststr, source, version in uploaded.copy():
-                if event.match(types=[mini_buildd.events.Type.INSTALLED, mini_buildd.events.Type.FAILED, mini_buildd.events.Type.REJECTED], distribution=diststr, source=source, version=version):
+            for upload in unfinished():
+                if event.match(types=[mini_buildd.events.Type.INSTALLED, mini_buildd.events.Type.FAILED, mini_buildd.events.Type.REJECTED], distribution=upload["distribution"], source=upload["source"], version=upload["version"]):
                     LOG.debug(f"Keyring package result: {event}")
-                    uploaded.remove((diststr, source, version))
+                    upload["event"] = event.type.name
                     if (event.type == mini_buildd.events.Type.INSTALLED) and not self.args["without_migration"].value():
-                        repository, distribution, suite = mini_buildd.mdls().repository.parse_diststr(diststr)
-                        repository.mbd_package_migrate(source, distribution, suite, full=True, version=version)
+                        repository, distribution, suite = mini_buildd.mdls().repository.parse_diststr(upload["distribution"])
+                        repository.mbd_package_migrate(upload["source"], distribution, suite, full=True, version=upload["version"])
 
 
 @_pimpdoc(PORT_RESULT_DOC)
@@ -1225,22 +1218,24 @@ class TestPackages(_Admin, _Running, _Confirm, Call):
         yield BoolArgument(["--with-check", "-c"], doc="Check for correct packager results")
 
     def run(self):
-        self.result["uploaded"] = []
+        self.result = []
         events = mini_buildd.events.Attach(mini_buildd.get_daemon().events)
 
         for source in self.args["sources"].value():
             for diststr in self.args["distributions"].value():
-                self.result["uploaded"].append(mini_buildd.package.upload_template_package(mini_buildd.package.TestPackage(source, auto_ports=self.args["auto_ports"].value()), diststr))
+                self.result.append(mini_buildd.package.upload_template_package(mini_buildd.package.TestPackage(source, auto_ports=self.args["auto_ports"].value()), diststr))
 
         if self.args["with_check"].value():
-            uploaded = set(self.result["uploaded"])
-            while uploaded:
+            def unfinished():
+                return [upload for upload in self.result if "event" not in upload]
+
+            while unfinished():
                 event = events.get()
-                for diststr, source, version in uploaded.copy():
-                    if event.match(types=[mini_buildd.events.Type.INSTALLED, mini_buildd.events.Type.FAILED, mini_buildd.events.Type.REJECTED], distribution=diststr, source=source, version=version):
-                        uploaded.remove((diststr, source, version))
-                        if (event.type == mini_buildd.events.Type.FAILED and source in ["mbd-test-archall", "mbd-test-cpp"]) or \
-                           (event.type == mini_buildd.events.Type.INSTALLED and source in ["mbd-test-ftbfs"]):
+                for upload in unfinished():
+                    if event.match(types=[mini_buildd.events.Type.INSTALLED, mini_buildd.events.Type.FAILED, mini_buildd.events.Type.REJECTED], distribution=upload["distribution"], source=upload["source"], version=upload["version"]):
+                        upload["event"] = event.type.name
+                        if (event.type == mini_buildd.events.Type.FAILED and upload["source"] in ["mbd-test-archall", "mbd-test-cpp"]) or \
+                           (event.type == mini_buildd.events.Type.INSTALLED and upload["source"] in ["mbd-test-ftbfs"]):
                             raise mini_buildd.HTTPBadRequest(f"Test package failed: {event}")
                         LOG.debug(f"Test package result: {event}")
 
