@@ -18,7 +18,7 @@ class Uploader(mini_buildd.threads.EventThread):
     def __init__(self):
         super().__init__(name="uploader")
 
-        self.failed = []
+        self.failed = {}
         for bres_entry in (f for f in mini_buildd.config.ROUTES["builds"].path.ifiles("", ".changes") if mini_buildd.changes.Buildresult.match(f.path)):
             LOG.debug(f"Checking buildresult: {bres_entry.path}")
             try:
@@ -34,23 +34,20 @@ class Uploader(mini_buildd.threads.EventThread):
             try:
                 event.upload(mini_buildd.net.ClientEndpoint(event.cget("Upload-To"), protocol="ftp"))
                 mini_buildd.get_daemon().events.log(mini_buildd.events.Type.BUILT, event)
-                if event in self.failed:
-                    self.failed.remove(event)
+                self.failed.pop(event.bkey, None)
             except BaseException as e:
                 mini_buildd.log_exception(LOG, f"Upload to {event.cget('Upload-To')} failed (will retry)", e)
-                if event not in self.failed:
-                    self.failed.append(event)
+                self.failed[event.bkey] = event
 
     def retry_failed(self):
         with self.lock:
-            failed = self.failed.copy()  # Note: afaik, copy() is atomic in cpython >= 3.5, but not generally
-        for bres in failed:
-            self.queue.put(bres)
+            for bres in self.failed.values():
+                self.queue.put(bres)
 
 
 class Build(mini_buildd.threads.DeferredThread):
     """
-    .. important:: If the constructor fails, no buildresult would be uploaded (and packaging would hang). Keep it simple && be sure this does not fail on 'normal' error conditions.
+    .. note:: If the constructor fails, no buildresult would be uploaded (and packaging would hang). Keep it simple && be sure this does not fail on 'normal' error conditions.
     """
 
     def __init__(self, limiter, breq, uploader_queue):
