@@ -27,10 +27,12 @@ class Call():
 
     >>> Call(["echo", "-n", "hallo"]).check().stdout
     'hallo'
-    >>> Call(["ls", "__no_such_file__"]).check()
+
+    >>> Call(["ls", "__no_such_file__"]).check()  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
-    mini_buildd.HTTPBadRequest: HTTP 400: Internal call failed
+    mini_buildd.HTTPBadRequest: Internal call failed (HTTP 400 Bad request syntax or unsupported method)
+
     >>> Call(["printf stdin; printf stderr >&2"], stderr=subprocess.STDOUT, shell=True).stdout
     'stdinstderr'
     """
@@ -63,7 +65,7 @@ class Call():
         # Convenience 'label' for log output
         self.label = f"{'#' if run_as_root else '?'} {call[0]}.."
 
-        LOG.info(f"Called with returncode {self.result.returncode}: {self._call2shell(self.call)}")
+        LOG.debug(f"Called with returncode {self.result.returncode}: {self._call2shell(self.call)}")
 
     @classmethod
     def _bos2str(cls, value, errors="replace"):
@@ -150,10 +152,10 @@ def call_with_retry(call, retry_max_tries=5, retry_sleep=1, retry_failed_cleanup
     In case retry_max_tries is reached, the error from the last try is raised.
 
     >>> call_with_retry(["/bin/true"])
-    >>> call_with_retry(["/bin/false"])
+    >>> call_with_retry(["/bin/false"])  # doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
       ...
-    mini_buildd.HTTPBadRequest: HTTP 400: Internal call failed
+    mini_buildd.HTTPBadRequest: Internal call failed (HTTP 400 Bad request syntax or unsupported method)
     """
     for t in range(retry_max_tries):
         try:
@@ -162,7 +164,7 @@ def call_with_retry(call, retry_max_tries=5, retry_sleep=1, retry_failed_cleanup
         except BaseException as e:
             if t >= retry_max_tries - 1:
                 raise
-            LOG.error(f"Retrying call in {retry_sleep} seconds [retry #{t}]: {e}")
+            LOG.warning(f"Retrying call in {retry_sleep} seconds [retry #{t}]: {e}")
             if retry_failed_cleanup is not None:
                 retry_failed_cleanup()
             time.sleep(retry_sleep)
