@@ -9,6 +9,7 @@ import debian.deb822
 import mini_buildd.config
 import mini_buildd.misc
 import mini_buildd.files
+import mini_buildd.sbuild
 import mini_buildd.dist
 import mini_buildd.net
 
@@ -27,7 +28,10 @@ class Base(debian.deb822.Changes):
 
         self.dfn = mini_buildd.files.DebianName(self["Source"], self["Version"])
 
-    @property
+    def __str__(self):
+        """Superclass would show all values multiline, so we need to replace with s.th. less invasive"""
+        return f"{self.get('Source')}_{self.get('Version')} ({self.get('Architecture')})"
+
     def file_name(self):
         return os.path.basename(self.file_path) if self.file_path is not None else None
 
@@ -45,23 +49,16 @@ class Base(debian.deb822.Changes):
                               "priority": "extra",
                               "name": os.path.basename(file_name)})
 
-    def save_as(self, file_path, gnupg=None):
-        """
-        Write to file (optionally signed).
-        """
-        LOG.info(f"Saving changes: {file_path}")
-        os.makedirs(os.path.dirname(file_path), exist_ok=True)  # Be sure base dir is always available
-        with open(file_path, "w+", encoding=mini_buildd.config.CHAR_ENCODING) as f:
-            f.write(self.dump())
-        if gnupg is not None:
-            LOG.info(f"Signing changes: {file_path}")
-            gnupg.sign(file_path)
-        self.file_path = file_path
+    def upload_file_path(self):
+        return os.path.splitext(self.file_path)[0] + ".upload"
+
+    def uploaded(self):
+        return os.path.exists(self.upload_file_path())
 
     def _upload(self, ftp_cls, endpoint, force=False):
         pre_upload_timestamp = mini_buildd.misc.Datetime.now()
 
-        upload = os.path.splitext(self.file_path)[0] + ".upload"
+        upload = self.upload_file_path()
         if not force and os.path.exists(upload):
             with mini_buildd.fopen(upload) as uf:
                 LOG.warning(f"FTP: '{upload}' exists (already uploaded to '{uf.read()}')")
@@ -73,16 +70,16 @@ class Base(debian.deb822.Changes):
                     ftp.prot_p()
                 ftp.login()
                 ftp.cwd("/incoming")
-                for fd in self.get_files() + [{"name": self.file_name}]:
+                for fd in self.get_files() + [{"name": self.file_name()}]:
                     f = fd["name"]
                     LOG.debug(f"FTP: Uploading file: '{f}'")
                     with open(os.path.join(os.path.dirname(self.file_path), f), "rb") as fi:
                         ftp.storbinary(f"STOR {f}", fi)
                 with mini_buildd.fopen(upload, "w") as fi:
-                    fi.write(f"{endpoint.geturl()}")
-                LOG.info(f"FTP: '{self.file_name}' uploaded to '{endpoint.geturl()}'.")
+                    fi.write(endpoint.geturl())
+                LOG.info(f"FTP: '{self.file_name()}' uploaded to '{endpoint.geturl()}'.")
 
-        return pre_upload_timestamp  # for events --since=
+        return pre_upload_timestamp  # for events --after=
 
     def upload(self, endpoint, force=False):
         return self._upload(ftplib.FTP_TLS if endpoint.is_ssl() else ftplib.FTP, endpoint, force=force)
@@ -94,6 +91,31 @@ class Base(debian.deb822.Changes):
     def untar(self, dir_path):
         mini_buildd.files.Tar.extract(self.file_path + ".tar", dir_path)
 
+    def save_as(self, file_path, gnupg=None):
+        """
+        Write to file (optionally signed)
+        """
+        LOG.debug(f"Saving changes: {file_path}")
+        os.makedirs(os.path.dirname(file_path), exist_ok=True)  # Be sure base dir is always available
+        with open(file_path, "w+", encoding=mini_buildd.config.CHAR_ENCODING) as f:
+            f.write(self.dump())
+        if gnupg is not None:
+            LOG.debug(f"Signing changes: {file_path}")
+            gnupg.sign(file_path)
+        self.file_path = file_path
+
+    def _move_to(self, path):
+        """
+        Move changes file along with all accompanying files (needs ``file_path`` to be non-None)
+        """
+        LOG.debug(f"Moving changes: '{self.file_path}'->'{path.full}'")
+        for fd in [{"name": self.file_name()}] + self.get_files():
+            f = fd["name"]
+            f_abs = os.path.join(os.path.dirname(self.file_path), f)
+            LOG.debug(f"Moving '{f}' to '{path.full}'")
+            os.rename(f_abs, os.path.join(path.full, f))
+        self.file_path = path.join(path.full, self.file_name())
+
 
 class Changes(Base):
     FILE_APX = ""
@@ -104,7 +126,7 @@ class Changes(Base):
     def match(cls, file_path):
         return re.compile(f"^.+{mini_buildd.files.DebianName.uext(cls.FILE_APX)}_[^_]+.changes$").match(file_path)
 
-    def __init__(self, initial_value, create_events=False, create_builds=False):
+    def __init__(self, initial_value):
         super().__init__(initial_value)
 
         for key, hint in self.MANDATORY.items():
@@ -119,8 +141,8 @@ class Changes(Base):
         #: Key to identify a 'Build' (Buildrequest, Buildresult).
         self.bkey = os.path.join(self.key, self["Architecture"])
 
-        self.events_path = mini_buildd.config.ROUTES["events"].path.new_sub([self.bkey], create=create_events)
-        self.builds_path = mini_buildd.config.ROUTES["builds"].path.new_sub([self.bkey], create=create_builds)
+        self.events_path = mini_buildd.config.ROUTES["events"].path.new_sub([self.bkey])
+        self.builds_path = mini_buildd.config.ROUTES["builds"].path.new_sub([self.bkey])
 
     @property
     def timecode(self):
@@ -141,7 +163,7 @@ class Changes(Base):
                 "timecode": self.timecode,
                 "key": self.key,
                 "bkey": self.bkey,
-                "changes": os.path.join(self.bkey, self.file_name) if self.file_name else None,
+                "changes": os.path.join(self.bkey, self.file_name()) if self.file_name() else None,
             },
             **{field.snake_name: value for field, value in self.fields().items() if field.is_cfield or field.snake_name in ["architecture", "maintainer", "changed_by"]},
         }
@@ -152,16 +174,23 @@ class Changes(Base):
             tar_path = file_path + ".tar"
             tar.save_as(tar_path)
             self.add_file(tar_path)
-        self.save_as(file_path, mini_buildd.get_daemon().model.mbd_gnupg())
+        self.save_as(file_path, mini_buildd.get_daemon().gnupg)
 
-    def move_to(self, path):
-        LOG.info(f"Moving changes: '{self.file_path}'->'{path.full}'")
-        path.create()
-        for fd in [{"name": self.file_name}] + self.get_files():
-            f = fd["name"]
-            f_abs = os.path.join(os.path.dirname(self.file_path), f)
-            LOG.debug(f"Moving '{f}' to '{path.full}'")
-            os.rename(f_abs, os.path.join(path.full, f))
+    def create_events(self):
+        self.events_path.create()
+        return self
+
+    def move_to_events(self):
+        self.create_events()
+        return self._move_to(self.events_path)
+
+    def create_builds(self):
+        self.builds_path.create()
+        return self
+
+    def move_to_builds(self):
+        self.create_builds()
+        return self._move_to(self.builds_path)
 
 
 class Upload(Changes):
@@ -218,6 +247,7 @@ class Upload(Changes):
             "auto-ports": CSV,
             "deb-build-options": Str,
             "deb-build-profiles": Str,
+            "add-depends": Str,
         }, **_OPTIONS_DEPRECATED}
 
         @classmethod
@@ -232,6 +262,8 @@ class Upload(Changes):
                     "  Contents of DEB_BUILD_OPTIONS environment for building (see https://www.debian.org/doc/debian-policy/ch-source.html#debian-rules-and-deb-build-options).\n"
                     "deb-build-profiles[<arch>]=<profile> <profile>...\n"
                     "  Contents of DEB_BUILD_PROFILES environment for building (see https://wiki.debian.org/BuildProfileSpec).\n"
+                    "add-depends[<arch>]=<dep>,<dep>...\n"
+                    "  List of additional dependencies (comma-separated, like in ``debian/control``) (see ``sbuild --add-depends``).\n"
                     )
 
         @classmethod
@@ -363,7 +395,7 @@ class Upload(Changes):
                     if not in_changes:
                         files_from_pool.append(p)
                         from_pool = True
-                        LOG.info(f"Buildrequest: File added from pool: {p}")
+                        LOG.debug(f"Buildrequest: File added from pool: {p}")
                 else:
                     raise mini_buildd.HTTPBadRequest(f"MD5 mismatch in uploaded dsc vs. pool: {f['name']}")
 
@@ -384,14 +416,14 @@ class Upload(Changes):
             })
 
             # Generate config files
-            config_path = path.new_sub([".config"], create=True)
+            config_path = path.new_sub([mini_buildd.sbuild.CONFIG_DIR], create=True)
             config_files = {
-                "apt_sources.list": repository.mbd_get_apt_build_sources_list(distribution, suite_option).get(),
-                "apt_preferences": repository.mbd_get_apt_build_preferences(distribution, suite_option, self.options.get("internal-apt-priority")),
-                "apt_keys": repository.mbd_get_apt_keys(distribution),
-                "ssl_cert": mini_buildd.http_endpoint().get_certificate(),
-                "chroot_setup_script": distribution.chroot_setup_script,
-                "sbuildrc_snippet": distribution.sbuildrc_snippet,
+                mini_buildd.sbuild.CONFIG_APT_SOURCES_LIST: repository.mbd_get_apt_build_sources_list(distribution, suite_option).get(),
+                mini_buildd.sbuild.CONFIG_APT_PREFERENCES: repository.mbd_get_apt_build_preferences(distribution, suite_option, self.options.get("internal-apt-priority")),
+                mini_buildd.sbuild.CONFIG_APT_KEYS: repository.mbd_get_apt_keys(distribution),
+                mini_buildd.sbuild.CONFIG_SSL_CERT: mini_buildd.http_endpoint().get_certificate(),
+                mini_buildd.sbuild.CONFIG_CHROOT_SETUP_SCRIPT: distribution.chroot_setup_script,
+                mini_buildd.sbuild.CONFIG_SBUILDRC_SNIPPET: distribution.sbuildrc_snippet,
             }
             for file_name, content in config_files.items():
                 with mini_buildd.fopen(config_path.join(file_name), "w") as f:
@@ -399,7 +431,7 @@ class Upload(Changes):
 
             # Generate tar
             tar = mini_buildd.files.Tar()
-            tar.add([config_path.join(file_name) for file_name in config_files], arcdir=".config")
+            tar.add([config_path.join(file_name) for file_name in config_files], arcdir=mini_buildd.sbuild.CONFIG_DIR)
             tar.add([self.dsc_file_path()])
             tar.add([os.path.join(os.path.dirname(self.dsc_file_path()), f["name"]) for f in dsc.get("Files", [])])
             tar.add(files_from_pool)
@@ -416,6 +448,7 @@ class Upload(Changes):
 
             buildrequest.cset("Deb-Build-Options", self.options.get("deb-build-options", ao.architecture.name, distribution.mbd_get_extra_option("Deb-Build-Options", "")))
             buildrequest.cset("Deb-Build-Profiles", self.options.get("deb-build-profiles", ao.architecture.name, distribution.mbd_get_extra_option("Deb-Build-Profiles", "")))
+            buildrequest.cset("Add-Depends", self.options.get("add-depends", ao.architecture.name, distribution.mbd_get_extra_option("Add-Depends", "")))
 
             buildrequest.cset("Lintian-Mode", self.check_mode("lintian", ao.architecture.name, distribution.lintian_mode).mode.name)
             buildrequest.cset("Lintian-Extra-Options", distribution.lintian_extra_options)
@@ -433,7 +466,7 @@ class Upload(Changes):
                     buildrequest_dict[ao.architecture.name] = buildrequest
                     break
                 except BaseException as e:
-                    mini_buildd.log_exception(LOG, f"Uploading to '{builder['incoming_url']}' failed", e)
+                    mini_buildd.log_exception(LOG, f"Uploading to '{builder['incoming_url']}' failed", e, level=logging.ERROR)
             if ao.architecture.name not in buildrequest_dict:
                 raise mini_buildd.HTTPUnavailable(f"No working builder found for {ao.architecture.name}/{self.dist.codename}")
         return buildrequest_dict
@@ -458,7 +491,7 @@ class Buildrequest(Changes):
         bres_path = self.dfn.changes(self["Architecture"], mini_buildd.changes.Buildresult.FILE_APX)
         if os.path.exists(bres_path):
             return Buildresult(bres_path)
-        return Buildresult({k: self[k] for k in ["Distribution", "Source", "Version", "Architecture"] + [mini_buildd.misc.CField(cfield).fullname for cfield in ["Timecode", "Builder", "Buildlog"]]})
+        return Buildresult({k: self[k] for k in ["Distribution", "Source", "Version", "Architecture"] + [mini_buildd.misc.CField(cfield).fullname for cfield in ["Timecode", "Builder", "Buildlog", "Upload-To"]]})
 
     def schroot_name(self):
         """
@@ -472,13 +505,6 @@ class Buildrequest(Changes):
 class Buildresult(Changes):
     FILE_APX = "mini-buildd-buildresult"
 
-    def needs_build(self):
-        return self.file_name is None
-
-    def needs_upload(self):
-        upload_path = self.builds_path.join(self.dfn.upload(self["Architecture"]))
-        return not os.path.exists(upload_path)
-
     def success(self, upload, distribution, ignore_checks):
         arch = self["Architecture"]
         status = self.cget("Sbuild-Status")
@@ -509,9 +535,23 @@ class Buildresult(Changes):
         return check_all() if status == "successful" else status == "skipped"
 
 
-def select(file_path):
-    if Buildresult.match(file_path):
-        return Buildresult(file_path)
-    if Buildrequest.match(file_path):
-        return Buildrequest(file_path)
-    return Upload(file_path)
+def incoming2queue(file_path):
+    LOG.info(f"Trying to queue incoming: {file_path}")
+    try:
+        if Buildresult.match(file_path):
+            changes = Buildresult(file_path)
+            changes.move_to_events()
+            mini_buildd.get_daemon().packager.queue.put(changes)
+        elif Buildrequest.match(file_path):
+            changes = Buildrequest(file_path).create_events()
+            changes.move_to_builds()
+            mini_buildd.get_daemon().builder.queue.put(changes)
+        else:  # User Upload
+            changes = Upload(file_path)
+            changes.move_to_events()
+            mini_buildd.get_daemon().packager.queue.put(changes)
+    except Exception as e:
+        # If it at least qualifies as Changes, still try error/event handling for user experience
+        changes = Changes(file_path)
+        changes.move_to_events()
+        mini_buildd.get_daemon().events.log(mini_buildd.events.Type.REJECTED, changes, e)
