@@ -1,5 +1,4 @@
 import threading
-import queue
 import logging
 
 import django.conf
@@ -33,12 +32,11 @@ class RemotesKeyring(mini_buildd.gnupg.TmpGnuPG):
 
     def __init__(self):
         super().__init__(tmpdir_options={"prefix": "gnupg-remotes-keyring-"})
-        our_pub_key = mini_buildd.models.daemon.get().mbd_get_pub_key()
-        if our_pub_key:
-            self.add_pub_key(our_pub_key)
+        if mini_buildd.get_daemon().gnupg.pub_key:
+            self.add_pub_key(mini_buildd.get_daemon().gnupg.pub_key)
         for r in mini_buildd.models.gnupg.Remote.mbd_get_active_or_auto_reactivate():
             self.add_pub_key(r.key)
-            LOG.info(f"Remote key added for '{r}': {r.key_long_id}: {r.key_name}")
+            LOG.debug(f"Remote key added for '{r}': {r.key_long_id}: {r.key_name}")
 
 
 class UploadersKeyring(mini_buildd.gnupg.TmpGnuPG):
@@ -59,43 +57,42 @@ class UploadersKeyring(mini_buildd.gnupg.TmpGnuPG):
                 LOG.warning(f"User '{u}' does not have an uploader profile (deliberately removed?): {e}")
 
             if uploader and uploader.mbd_is_active() and uploader.may_upload_to.all().filter(identity=repo_identity):
-                LOG.info(f"Adding uploader key for '{repo_identity}': {uploader.key_long_id}: {uploader.key_name}")
+                LOG.debug(f"Adding uploader key for '{repo_identity}': {uploader.key_long_id}: {uploader.key_name}")
                 self.add_pub_key(uploader.key)
 
         # Add configured extra keyrings
         for line in r.extra_uploader_keyrings.splitlines():
             line = line.strip()
             if line and line[0] != "#":
-                LOG.info(f"Adding keyring: {line}")
+                LOG.debug(f"Adding keyring: {line}")
                 self.add_keyring(line)
 
         # Always add our key too for internal builds
-        our_pub_key = mini_buildd.models.daemon.get().mbd_get_pub_key()
-        if our_pub_key:
-            self.add_pub_key(our_pub_key)
+        if mini_buildd.get_daemon().gnupg.pub_key:
+            self.add_pub_key(mini_buildd.get_daemon().gnupg.pub_key)
 
 
 START_STOP_LOCK = threading.Lock()
 
 
-class Daemon(mini_buildd.threads.EventThread, metaclass=mini_buildd.misc.Singleton):
+class Daemon(mini_buildd.threads.PollerThread, metaclass=mini_buildd.misc.Singleton):
+    #: Schedule to wake up regularly (to do run-time maintenance)
+    _POLL_SLEEP = 120   # seconds
+
     @property
     def model(self):
         return mini_buildd.models.daemon.get()
 
     def __init__(self):
-        # Vars that are (re)generated when the daemon model is updated
-        self.incoming_queue = queue.Queue()
-
         self.builder = mini_buildd.builder.Builder(max_parallel_builds=self.model.build_queue_size)
         self.packager = mini_buildd.packager.Packager()
-        self.ftpd = mini_buildd.ftpd.FtpD(endpoint=self.model.mbd_get_ftp_endpoint(), queue=self.incoming_queue, handler_options=self.model.ftpd_options)
+        self.ftpd = mini_buildd.ftpd.FtpD(endpoint=self.model.mbd_get_ftp_endpoint(), handler_options=self.model.ftpd_options)
         self.crontab = mini_buildd.cron.Tab()
+        super().__init__(subthreads=[self.ftpd, self.packager, self.builder, self.crontab], sleep=self._POLL_SLEEP, name="daemon")
 
-        super().__init__(subthreads=[self.ftpd, self.packager, self.builder, self.crontab], events=self.incoming_queue, name="daemon")
-
-        # Events
-        self.events_queue = mini_buildd.events.Queue(mini_buildd.events.load(), maxlen=self.model.show_last_packages)
+        self.events = mini_buildd.events.Queue.load(maxlen=self.model.show_last_packages)
+        self.gnupg = self.model.mbd_gnupg()
+        self.public_key_cache = mini_buildd.gnupg.PublicKeyCache()
 
         #: Dict of functions to acquire possible attention strings (HTML support in 'main_menu_item.html' include).
         self.attention = {
@@ -110,7 +107,7 @@ class Daemon(mini_buildd.threads.EventThread, metaclass=mini_buildd.misc.Singlet
         return f"{'UP' if self.is_alive() else 'DOWN'}: {self.model}"
 
     def handshake_message(self):
-        return self.model.mbd_gnupg().gpgme_sign(mini_buildd.config.HANDSHAKE_MESSAGE)
+        return self.gnupg.gpgme_sign(mini_buildd.config.HANDSHAKE_MESSAGE)
 
     def sync(self):
         self.model.refresh_from_db()
@@ -118,14 +115,12 @@ class Daemon(mini_buildd.threads.EventThread, metaclass=mini_buildd.misc.Singlet
 
     def join(self, timeout=None):
         super().join()
-        self.events_queue.shutdown()
+        self.events.shutdown()
         self.sync()
 
-    def run_event(self, event):
-        if mini_buildd.changes.Buildrequest.match(event):
-            self.builder.queue.put(event)
-        else:
-            self.packager.queue.put(event)  # User upload or build result: packager
+    def run_poller(self):
+        LOG.debug("Daemon: Run-time maintenance...")
+        self.builder.uploader.retry_failed()
 
     def check(self):
         """Global check -- check Daemon instance and implicitly all it's dependencies."""
@@ -145,6 +140,7 @@ class Daemon(mini_buildd.threads.EventThread, metaclass=mini_buildd.misc.Singlet
             if self.is_alive():
                 self.shutdown()
                 self.join()
+                self.public_key_cache.close()
                 Daemon.destroy()
 
 
