@@ -6,6 +6,7 @@ import datetime
 import glob
 import json
 import threading
+import itertools
 import collections
 import functools
 import weakref
@@ -18,27 +19,6 @@ import debian.debian_support
 LOG = logging.getLogger(__name__)
 
 
-class _NoLock():
-    """
-    Debug/temporary only: Experienced dead locks -- not reliably reproducible for now.
-
-    May-be-related to our locking here. Use this to exclude our locking once we can
-    reproduce the lockdown.
-    """
-
-    def acquire(self):
-        pass
-
-    def release(self):
-        pass
-
-    def __enter__(self):
-        pass
-
-    def __exit__(self, _type, _value, _traceback):
-        pass
-
-
 class Type(enum.Enum):
     REJECTED = enum.auto()
 
@@ -53,6 +33,22 @@ class Type(enum.Enum):
     REMOVED = enum.auto()
 
 
+#: Human-readable semantics of all event types.
+DESC = {
+    Type.REJECTED: "User upload was rejected",
+
+    Type.BUILDING: "Building on one of our chroots has started",
+    Type.BUILT: "Building on one of our chroots has finished (does not imply a successful build result)",
+
+    Type.PACKAGING: "Packaging to one of our repositories has started",
+    Type.INSTALLED: "Packaging to one of our repositories was successful",
+    Type.FAILED: "Packaging to one of our repositories has failed",
+
+    Type.MIGRATED: "A package in one of our repositories has been migrated",
+    Type.REMOVED: "A package in one of our repositories has been removed",
+}
+
+
 class Event():
     def __init__(self, type_, distribution, source, version, extra):
         self.type = type_
@@ -62,10 +58,16 @@ class Event():
         self.timestamp = mini_buildd.misc.Datetime.now()
         self.extra = extra
 
+        #: Special run-time only flag (see Events.log()). For events outside Events(), this will always be false.
+        self.ongoing = False
+
     def strerror(self):
         """Public human-readable one-liner string from ``extra.error``."""
         error = self.extra.get("error")  # Rfc7807 dict
-        return "OK" if error is None else error.get("detail", "No details")
+        return "" if error is None else error.get("detail", "No details")
+
+    def desc(self):
+        return DESC[self.type]
 
     def __str__(self):
         return ": ".join(filter(None, [f"{self.type.name}({self.distribution}): {self.source}-{self.version}", self.strerror()]))
@@ -139,13 +141,11 @@ class Event():
 
 
 class Queue(collections.deque):
-    MAX_CLIENTS = 100
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._clients = {}
         self._lock = threading.Lock()
-        self.running = collections.OrderedDict()
+        self.ongoing = collections.OrderedDict()
 
     def log(self, typ, changes, exception=None, extra=None):
         if exception is None and changes.cget("Internal-Error"):
@@ -161,35 +161,40 @@ class Queue(collections.deque):
 
             for _key, (weakref_obj, q) in self._clients.items():
                 if weakref_obj() is not None:
-                    LOG.info(f"Queuing event for object '{weakref_obj}'")
+                    LOG.debug(f"Queuing event for object '{weakref_obj}'")
                     q.put(event)
 
-            # Handle "running" dict
+            # Handle "ongoing" dict
             if event.type in [Type.PACKAGING, Type.BUILDING]:
-                LOG.debug(f"Adding event to 'running': {changes.bkey}={event}")
-                self.running[changes.bkey] = event
+                LOG.debug(f"Adding event to 'ongoing': {changes.bkey}={event}")
+                self.ongoing[changes.bkey] = event
+                event.ongoing = True
             elif event.type in [Type.BUILT, Type.INSTALLED, Type.FAILED]:
-                LOG.debug(f"Deleting event from 'running': {changes.bkey} from {event}")
-                self.running.pop(changes.bkey, None)   # pop: Should rather not raise exception here if key is not there for some reason
+                LOG.debug(f"Deleting event from 'ongoing': {changes.bkey} from {event}")
+                ongoing_event = self.ongoing.pop(changes.bkey, None)   # None: Should rather not raise exception here if key is not there for some reason
+                if ongoing_event is not None:
+                    ongoing_event.ongoing = False
 
-    def attach(self, obj, since=None):
+    def attach(self, obj, after=None):
         with self._lock:
             # Garbage collection
             for key in [key for key, (weakref_obj, _q) in self._clients.items() if weakref_obj() is None]:
-                LOG.info(f"Detach event queue from object '{key}': Dead.")
                 del self._clients[key]
+                LOG.debug(f"Events queue client detached ({len(self._clients)}/{mini_buildd.config.MAX_EVENTS_QUEUE_CLIENTS}): {obj}")
 
             if id(obj) not in self._clients:
-                if len(self._clients) >= self.MAX_CLIENTS:
-                    raise mini_buildd.HTTPUnavailable(f"Max client limit ({self.MAX_CLIENTS}) exceeded")
-                LOG.info(f"Events queue: Attaching client: {obj}")
+                if len(self._clients) >= mini_buildd.config.MAX_EVENTS_QUEUE_CLIENTS:
+                    raise mini_buildd.HTTPUnavailable(f"Max client limit ({mini_buildd.config.MAX_EVENTS_QUEUE_CLIENTS}) exceeded")
                 self._clients[id(obj)] = (weakref.ref(obj), queue.Queue())
-                if since:
+                LOG.debug(f"Events queue client attached ({len(self._clients)}/{mini_buildd.config.MAX_EVENTS_QUEUE_CLIENTS}): {obj}")
+
+                # Optionally put past events
+                if after is not None:
                     for event in reversed(self):
-                        if event.timestamp >= since:
+                        if event.timestamp > after:
                             self._clients[id(obj)][1].put(event)
 
-        return self._clients[id(obj)][1]
+            return self._clients[id(obj)][1]
 
     def shutdown(self):
         """Hint shutdown to all client (queues). Essentially makes blocking get() in httpd.py continue so httpd can shutdown."""
@@ -198,7 +203,7 @@ class Queue(collections.deque):
                 try:
                     client[1].put_nowait(mini_buildd.config.SHUTDOWN)
                 except Exception as e:
-                    LOG.info(f"Events queue: Ignoring failure to hint shutdown to client {client[0]}: {e}")
+                    LOG.warning(f"Events queue: Ignoring failure to hint shutdown to client {client[0]}: {e}")
 
     def to_json(self):
         return [event.to_json() for event in self]
@@ -207,6 +212,15 @@ class Queue(collections.deque):
     def from_json(cls, events, maxlen):
         return Queue([Event.from_json(event) for event in events], maxlen=maxlen)
 
+    @classmethod
+    def load(cls, maxlen):
+        return Queue(load(maxlen=maxlen), maxlen=maxlen)
+
+    def mbd_slice(self, maxlen=None):
+        """Return triple: <list of newest events>, <total number of events in queue>, <max number of events in queue>, <number of ongoing>."""
+        with self._lock:
+            return list(itertools.islice(self, maxlen)), len(self), self.maxlen, len(self.ongoing)
+
 
 class Attach():
     def __init__(self, events, *args, **kwargs):
@@ -226,6 +240,6 @@ def ifilter(ievents, types=None, distribution=None, source=None, version=None, m
                 break
 
 
-def load(path="", since=None):
+def load(path="", after=None, before=None, maxlen=None):
     """Load events (as normal list) from events path, ordered as issued using event's timestamp."""
-    return sorted([Event.load(file_path) for stamp, file_path in mini_buildd.config.ROUTES["events"].path.ifiles(path, ".json", since)], key=operator.attrgetter("timestamp"), reverse=True)
+    return sorted([Event.load(entry.path) for entry in mini_buildd.config.ROUTES["events"].path.ifiles(path, ".json", after=after, before=before)], key=operator.attrgetter("timestamp"), reverse=True)[0:maxlen]
