@@ -59,7 +59,7 @@ class ExceptionMiddleware():
     def error(cls, request, exception, output="html", status=http.HTTPStatus.INTERNAL_SERVER_ERROR):
         http_exception = mini_buildd.e2http(exception, status)
         if http_exception.status != http.HTTPStatus.OK:
-            LOG.error(f"Middleware exception: {exception}")
+            mini_buildd.log_exception(LOG, "Middleware exception", exception)
             mbd_message(request, exception)
 
         return django.template.response.TemplateResponse(request,
@@ -348,4 +348,4 @@ class APIView(TemplateView):
         # API call output
         if self.template_name is not None:
             return super().get(request, *args, **kwargs)
-        return django.http.JsonResponse(self.mbd_call.result)
+        return django.http.JsonResponse(self.mbd_call.result, safe=False)
